package it.tidalwave.geo.viewer.worldwindprovider.impl;

import javax.annotation.Nonnull;
import gov.nasa.worldwind.geom.Angle;
import gov.nasa.worldwind.geom.Sector;

public class MercatorSector extends Sector
  {
    @Nonnull
    private final double minLatPercent;

    @Nonnull
    private final double maxLatPercent;

    public MercatorSector (final double minLatPercent,
                           final double maxLatPercent,
                           final @Nonnull Angle minLongitude,
                           final @Nonnull Angle maxLongitude)
      {
        super(gudermannian(minLatPercent), gudermannian(maxLatPercent), minLongitude, maxLongitude);
        this.minLatPercent = minLatPercent;
        this.maxLatPercent = maxLatPercent;
      }

    @Nonnull
    public static MercatorSector fromDegrees (final double minLatPercent,
                                              final double maxLatPercent,
                                              final double minLongitude,
                                              final double maxLongitude)
      {
        return new MercatorSector(minLatPercent,
                                  maxLatPercent,
                                  Angle.fromDegrees(minLongitude),
                                  Angle.fromDegrees(maxLongitude));
      }

    @Nonnull
    public static MercatorSector fromSector (final @Nonnull Sector sector)
      {
        return new MercatorSector(gudermannianInverse(sector.getMinLatitude()),
                                  gudermannianInverse(sector.getMaxLatitude()),
                                  new Angle(sector.getMinLongitude()),
                                  new Angle(sector.getMaxLongitude()));
      }

    public static double gudermannianInverse (final @Nonnull Angle latitude)
      {
        //see http://en.wikipedia.org/wiki/Mercator_projection
        return Math.log(Math.tan(Math.PI / 4.0 + latitude.radians / 2.0)) / Math.PI;
      }

    @Nonnull
    public static Angle gudermannian (final double percent)
      {
        //see http://en.wikipedia.org/wiki/Mercator_projection
        return Angle.fromRadians(Math.atan(Math.sinh(percent * Math.PI)));
      }

    public double getMinLatPercent()
      {
        return minLatPercent;
      }

    public double getMaxLatPercent()
      {
        return maxLatPercent;
      }
  }
